/* FILE NAME: server.c */
#include <stdio.h>
#include "arith.h"                    /* header created by the idl compiler */
#include "status.h"                   /* header with the CHECK_STATUS macro */

main ()
{
   unsigned long        status;                             /* error status */
   rpc_binding_vector_t *binding_vector;          /* set of binding handles */
   unsigned char        *entry_name;         /* entry name for name service */

   status =                                                 /* error status */
   RpcServerRegisterIf(        /* register interface with the RPC runtime   */
      arith_v0_0_s_ifspec,             /* interface specification (arith.h) */
      NULL,
      NULL
   );
   CHECK_STATUS(status, "Can't register interface", ABORT);

   status =
   RpcServerUseAllProtseqs(                  /* create binding information  */
      RPC_C_PROTSEQ_MAX_REQS_DEFAULT,              /* queue size for calls  */
      NULL                               /* no security descriptor is used  */
   );
   CHECK_STATUS(status, "Can't create binding information", ABORT);

   status =
   RpcServerInqBindings(      /* obtain this server's binding information   */
      &binding_vector
   );
   CHECK_STATUS(status, "Can't get binding information", ABORT);

   entry_name = (unsigned char *)getenv("ARITHMETIC_SERVER_ENTRY");
   status =
   RpcNsBindingExport(          /* export entry to name service database    */
      RPC_C_NS_SYNTAX_DEFAULT,                /* syntax of the entry name   */
      entry_name,                  /* entry name for name service           */
      arith_v0_0_s_ifspec,              /* interface specification (arith.h)*/
      binding_vector,              /* the set of server binding handles     */
      NULL
   );
   CHECK_STATUS(status, "Can't export to name service database", ABORT);

   status =
   RpcEpRegister(             /* register endpoints in local endpoint map   */
      arith_v0_0_s_ifspec,             /* interface specification (arith.h) */
      binding_vector,             /* the set of server binding handles      */
      NULL,
      NULL
   );
   CHECK_STATUS(status, "Can't add address to the endpoint map", ABORT);

   status =
   RpcBindingVectorFree(            /* free set of server binding handles   */
      &binding_vector
   );
   CHECK_STATUS(status, "Can't free binding handles and vector", ABORT);

   puts("Listening for remote procedure calls...");
   status =
   RpcServerListen(                           /* listen for remote calls    */
      1,                                       /* minimum number of threads */
      RPC_C_LISTEN_MAX_CALLS_DEFAULT,          /*concurrent calls to server */
      NULL                   /* continue listening until explicitly stopped */
   );
   CHECK_STATUS(status, "rpc listen failed", ABORT);
}

/*************************************************************************/
/***                MIDL_user_allocate / MIDL_user_free                ***/
/*************************************************************************/

void * __RPC_API
MIDL_user_allocate
           (
            size
           )
size_t size;
{

    unsigned char * ptr;
    ptr = malloc( size );
    return ( (void *)ptr );

}

void __RPC_API
MIDL_user_free
           (
            object
           )
void * object;
{
    free (object);
}
