/*
**
**
** MODULE: wclient.c
**
**
** PROGRAM: Windows wphnbk application 
**
**
**          
**
**
*/      

#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "phnbk.h"
#include "wphnbk.h"



int            lookup_status;     /* lookup return status          */
error_status_t status;            /* rpc status                    */
unsigned char  input[LINESIZE];   /* 'find' search string          */
char           output[LINESIZE];  /* string returned from database */
char           oldmatch[LINESIZE];/* previous 'find' string        */
unsigned char  server[80];        /* string binding for server     */
short          operation;         /* operation requested           */
short          no_handle;         /* handle not initialized flag   */
unsigned char  hostname[32];      /* phnbk server host name        */

long FAR PASCAL WndProc (HWND, WORD, WORD, LONG) ;

int 
PASCAL WinMain 
              (
               HANDLE hInstance, 
               HANDLE hPrevInstance,
               LPSTR lpszCmdLine, 
               int nCmdShow
              )
{
    char szAppName [] = "WPHNBK" ;
    HWND        hwnd ;
    MSG         msg;
    WNDCLASS    wndclass ;

    /*
    ** Initialize strings
    */
    input[0]    = '\0';
    output[0]   = '\0';
    oldmatch[0] = '\0';
    server[0]   = '\0';
    hostname[0] = '\0';

    no_handle = TRUE;

    /*
    ** Standard Windows stuff...
    */
    if (!hPrevInstance)
        {
        wndclass.style          = CS_HREDRAW | CS_VREDRAW;
        wndclass.lpfnWndProc    = (WNDPROC) WndProc ;
        wndclass.cbClsExtra     = 0 ;
        wndclass.cbWndExtra     = DLGWINDOWEXTRA ;
        wndclass.hInstance      = hInstance ;
        wndclass.hIcon          = LoadIcon(hInstance,szAppName);
        wndclass.hCursor        = LoadCursor ((HINSTANCE)NULL, IDC_ARROW) ;
        wndclass.hbrBackground  = (HBRUSH) (COLOR_WINDOW + 1) ;
        wndclass.lpszMenuName   = NULL ;
        wndclass.lpszClassName  = szAppName ;

        RegisterClass (&wndclass) ;
        }

    hwnd = CreateDialog (hInstance, szAppName, 0, NULL) ;

    ShowWindow (hwnd, nCmdShow) ;

    SetFocus ( GetDlgItem (hwnd, HOSTNAMEBOX ) );

    /*
    ** Start accepting messages
    */
    while ( GetMessage (&msg, NULL, 0, 0) )
        {
        TranslateMessage (&msg) ;
        DispatchMessage (&msg) ;
        }

    return msg.wParam ;
}

short
InitHandle
          (
           HWND hwnd
          )
{

    /*
    ** Read server host name
    */
    GetDlgItemText (hwnd, HOSTNAMEBOX, hostname, 16 );
    
    /*
    ** Warn user if they haven't specified a host name
    */
    if (hostname[0] == '\0')
        {
        MessageBox 
                  (
                   hwnd,
                   "Please enter server host name",
                   "ERROR", 
                   MB_OK
                  );
        SetFocus ( GetDlgItem (hwnd, HOSTNAMEBOX) );
        return (-1);
        }
    
    /*
    ** Build server string binding
    */
    strcat (server, "ncacn_ip_tcp:");
    strcat (server, hostname);
    /*
    ** Convert the character string binding into an RPC handle
    */
    status = RpcBindingFromStringBinding
               (
                server, 
                &xhandle 
               );

    if (status)
        {         
        MessageBox
                  (
                   hwnd,
                   "Invalid string binding",
                   "ERROR",
                   MB_OK
                  );
        exit (EXIT_FAILURE);
        }
    
    no_handle = FALSE;

    return (0);

}


void 
ShowResult 
          (
           HWND hwnd
          )
{
    /*
    ** Display lookup results, based on the context of
    **    the requested operation
    */
    if (operation == BROWSE)
        {
        /*
        ** BROWSE -- return next entry
        */
        if (lookup_status == NORMAL)
            /*
            ** Everything ok, display next entry
            */
            SetDlgItemText (hwnd,RESULTSBOX,output);
        else
            {
            /*
            ** Otherwise, we hit end of file...
            */
            SetDlgItemText (hwnd,RESULTSBOX,"");
            SetDlgItemText (hwnd,INFOBOX,"No more entries");
            }
        }
    else
        {
        /*
        ** Operation was a Find or Find Next...tailor message
        **    syntax to reflect the operation.
        */
        if (lookup_status == NORMAL)
            {
            /*
            ** Print results
            */
            SetDlgItemText (hwnd,RESULTSBOX,output);
         
            /*
            ** Determine if this was first match, or subsequent match
            */
            if (operation == FIRSTMATCH)
                SetDlgItemText (hwnd,INFOBOX,"Match found");
            else
                SetDlgItemText (hwnd,INFOBOX,"Another match found");
            }
        else
            /*
            ** Hit end of file during search
            */
            if (operation == FIRSTMATCH)
                SetDlgItemText (hwnd,INFOBOX,"Match not found");
            else
                SetDlgItemText (hwnd,INFOBOX,"No other matches found");
        }

    return;
    }


long 
FAR PASCAL WndProc 
                  (
                   HWND hwnd, 
                   WORD message, 
                   WORD wParam, 
                   LONG lParam
                  )
    {
    /*
    ** We switch cursors to the hourglass during
    **    a lookup RPC.  This is for saving the
    **    regular pointer.
    */
    HCURSOR OldCursor;


    /*
    ** First thing, save the match string from last time around
    */
    strcpy (oldmatch, input);

    /*
    ** Switch on the incoming message type (standard Windows
    **    programming)
    */
    switch (message)
        {
        /*
        ** Got a button pushed
        */
        case WM_COMMAND:

            switch (wParam)
                {
                /*
                ** Either a Find or a Find Next
                */
                case FINDBUTTON:
                    if (no_handle) 
                        if (InitHandle(hwnd)) break;
                    /*
                    ** Clear current text
                    */
                    SetDlgItemText(hwnd,RESULTSBOX,"");
                    SetDlgItemText(hwnd,INFOBOX,"");

                    /*
                    ** Read the search string
                    */
                    GetDlgItemText(hwnd,SEARCHBOX,input,32);

                    /*
                    ** Make sure user entered a search string
                    */
                    if (input[0] == (unsigned char) '\0')
                        {
                        MessageBox 
                                  (
                                   hwnd,
                                   "Missing Search String!",
                                   "ERROR", 
                                   MB_OK
                                  );
                        /*
                        ** Set focus back to SEARCHBOX so user can
                        **    enter search string
                        */
                        SetFocus ( GetDlgItem (hwnd, SEARCHBOX) );
                        }
                    else
                        {
                        /*
                        ** Search string is present. Save existing 
                        **    pointer and display hourglass
                        */
                        OldCursor = SetCursor (LoadCursor(NULL,IDC_WAIT));
                        ShowCursor (TRUE);

                        /*
                        ** Determine desired operation
                        */
                        if (strcmp(oldmatch,input))
                            operation = FIRSTMATCH;
                        else
                            operation = NEXTMATCH;

                        /*
                        ** Perform the requested operation
                        */
                        lookup_status = lookup
                                              (
                                               operation,
                                               input,
                                               output
                                              );

                        /*
                        ** Restore pointer cursor
                        */
                        ShowCursor (FALSE);
                        SetCursor ( OldCursor );
       
                        /*
                        ** Display lookup results
                        */
                        ShowResult(hwnd);
                        }
                    
                    break;

                /*
                ** BROWSE -- return next entry
                */
                case BROWSEBUTTON:
                    if (no_handle) 
                        if(InitHandle(hwnd)) break;
                    /*
                    ** Clear existing text and display status
                    */
                    SetDlgItemText(hwnd,RESULTSBOX,"");
                    SetDlgItemText(hwnd,SEARCHBOX,"");
                    SetDlgItemText(hwnd,INFOBOX,"Browsing...");

                    /*
                    ** Switch to hourglass cursor
                    */
                    OldCursor = SetCursor (LoadCursor(NULL,IDC_WAIT));
                    ShowCursor (TRUE);

                       
                    operation = BROWSE;

                    /*
                    ** Perform the requested operation
                    */
                    lookup_status = lookup 
                                          ( 
                                           operation, 
                                           input,
                                           output
                                          );

                    /*
                    ** Restore pointer cursor
                    */
                    ShowCursor (FALSE);
                    SetCursor ( OldCursor );
                       
                    /*
                    ** Display operation results
                    */
                    ShowResult(hwnd);
                       
                    break;

                /*
                ** User has requested a RESET.  This clears all
                **    text and rewinds the phonebook file
                */
                case RESETBUTTON:
                    if (no_handle) 
                        if (InitHandle(hwnd)) break;
                    /*
                    ** Clear all text
                    */
                    SetDlgItemText(hwnd,RESULTSBOX,"");
                    SetDlgItemText(hwnd,INFOBOX,"");
                    SetDlgItemText(hwnd,SEARCHBOX,"");

                    input[0] = '\0';

                    operation = RESET;

                    /*
                    ** Perform the requested operation
                    */
                    lookup_status = lookup 
                                          ( 
                                           operation, 
                                           input,
                                           output
                                          );

                    break;

                }

            return 0 ;

        /*
        ** User has closed the application
        */
        case WM_DESTROY:
            if (!no_handle)
                {
                /*
                ** Free binding handle, post quit message and leave
                */
                status = RpcBindingFree
                           (
                            &xhandle
                           );
                }

            PostQuitMessage (0) ;
            return 0 ;

        /*
        ** Ignore other messages
        */
        default:
            return DefWindowProc (hwnd, message, wParam, lParam) ;
        }
}
